const state = {
    loadingStores: false,
    stores: [],
    selectedStore: null
}

const getters = {
    loadingStores(state) {
        return state.loadingStores
    },
    stores(state) {
        return state.stores
    },
    selectedStore(state) {
        return state.selectedStore || { name: null }
    },
    hasSelectedStore(state) {
        return !!state.selectedStore
    },
    showStoreSelector(state) {
        return state.stores.length > 1 
    }
}

const mutations = {
    setLoadingStores(state, loading) {
        state.loadingStores = !!loading
    },
    setStores(state, stores) {
        state.stores = stores
    },
    setSelectedStore(state, storeId) {
        if (!storeId) {
            state.selectedStore = null
        }

        state.selectedStore = state.stores.find(s => s.id == storeId)
    }
}

const actions = {
    initStores({ commit }, stores) {
        commit('setStores', stores)
        commit('setLoadingStores')
    },
    resetStore({ commit }) {
        commit('setLoadingStores')
        commit('setStores', [])
    },
    selectStore({ commit }, storeId) {
        commit('setSelectedStore', storeId)
    }
}

export default {
    state,
    getters,
    actions,
    mutations
}